import os
from abstract_utilities import *
from abstract_solcatcher_database import get_timestamp_from_data

class PathManager:
    @staticmethod
    def get_abs_path():
        return os.path.abspath(__file__)

    @staticmethod
    def get_abs_dir():
        return os.path.dirname(PathManager.get_abs_path())

    @staticmethod
    def create_abs_path(relative_path):
        return os.path.join(PathManager.get_abs_dir(), relative_path)

    @staticmethod
    def search_for_file(filename, base_dirs):
        for base in base_dirs:
            for root, _, files in os.walk(base):
                if filename in files:
                    return os.path.join(root, filename)
        return None

    @staticmethod
    def get_conversation_path():
        filename = 'conversations.json'
        # Priority: abs dir, cwd, ~/Documents
        abs_dir = PathManager.get_abs_dir()
        cwd = os.getcwd()
        docs = os.path.expanduser('~/Documents')

        # Try non-recursive checks first
        for location in [abs_dir, cwd, docs]:
            candidate = os.path.join(location, filename)
            if os.path.isfile(candidate):
                return candidate

        # Fall back to recursive search in ~/Documents
        found_path = PathManager.search_for_file(filename, [docs])
        if found_path:
            return found_path

        raise FileNotFoundError(f"'{filename}' not found in script dir, CWD, or ~/Documents.")


def get_conversation_data(file_path=None):
    file_path = get_convo_path(file_path=file_path)
    return safe_read_from_json(file_path)


def is_valid_time(comp_timestamp, timestamp=None, before=True):
    if timestamp is None:
        return True
    return (timestamp >= comp_timestamp) if before else (timestamp <= comp_timestamp)


def find_for_string(string, parts):
    return [part for part in parts if string.lower() in str(part).lower()]


def is_strings_in_string(strings, parts):
    if not strings:
        return parts
    strings = make_list(strings)
    for string in strings:
        parts = find_for_string(string, parts)
        if not parts:
            return []
    return parts


def get_parts(data):
    parts = []
    for path in find_paths_to_key(json_data=data, key_to_find="message"):
        message_value = get_value_from_path(data, path)
        for sub_path in find_paths_to_key(message_value, 'parts'):
            parts_value = get_value_from_path(message_value, sub_path)
            parts.extend([p for p in parts_value if p])
    return parts
def get_convo_path(file_path=None):
    file_path = file_path or PathManager.get_conversation_path()
    return file_path
class ConversationManager(metaclass=SingletonMeta):
    def __init__(self,file_path=None):
        
        if not hasattr(self, 'initialized') or self.initialized == False:
            self.initialized = True
            self.original_file_path = file_path
            self.convo_path = get_convo_path(file_path=file_path)
            self.conversation_data = get_conversation_data(self.convo_path)
        
def get_conversation_manager(file_path=None):
    conversation_mgr = ConversationManager(file_path=file_path)
    if conversation_mgr.original_file_path != file_path:
        conversation_mgr.initialized = False
        conversation_mgr = ConversationManager(file_path=file_path)
    return conversation_mgr

def get_convo_data(file_path=None):
    conversation_mgr = get_conversation_manager(file_path=file_path)
    return conversation_mgr.conversation_data

def search_in_conversation(strings=None, *args, **kwargs):
    strings = make_list(strings)
    timestamp = get_timestamp_from_data(kwargs)
    before = kwargs.get('before', True)
    file_path = kwargs.get('file_path', None)
    results = []
    for convo in get_convo_data(file_path=file_path):
        create_time = make_list(get_any_value(convo, 'create_time'))[0]
        if is_valid_time(comp_timestamp=create_time, timestamp=timestamp, before=before):
            parts = get_parts(convo)
            matched_parts = is_strings_in_string(strings, parts)
            if matched_parts:
                results.append(matched_parts)
    return results



from abstract_ai import *
from abstract_database import *
from abstract_security import *
from conn_manager import *
def get_raw_response_directory():
    return os.path.join(os.getcwd(),"response_data")
def get_latest_file(directory):
    files = glob.glob(f"{directory}/*")
    latest_file = max(files, key=os.path.getmtime)
    return latest_file
def get_last_created_response_file_path():
    raw_response_directory = get_raw_response_directory()
    return get_latest_file(raw_response_directory)
def read_previous_query():
    recent_file_path = get_last_created_response_file_path()
    return safe_read_from_json(recent_file_path)
def parse_previous_query():
    response_mgr = ResponseFileCollator()
    return self.ordered_data[0]
def get_db_query():
    file_path = get_last_created_response_file_path()
    return generate_query_from_recent_response(file_path)
def get_db_values(dbName,dbType):
    keys = {"user":None,"dbname":None,"host":None,"port":None,"password":None}
    for key,value in keys.items():
        env_key = get_db_key(dbName,dbType,key)
        keys[key]=get_env_key_value(env_key)
    return keys
def get_db_key(dbName,dbType,extra=None):
    key = f"{dbName.upper()}_{dbType.upper()}"
    if extra:
        key+=f"_{extra.upper()}"
    return key
def get_env_key_value(env_key):
    return get_env_value(env_key,deep_scan=True)
def search_env_path(dbName,dbType):
    return get_env_path(get_db_key(dbName=dbName,dbType=dbType),deep_scan=True)
def get_connection_manager(dbName,dbType):
    env_path = search_env_path(dbName=dbName,dbType=dbType)
    return connectionManager(dbName=dbName,dbType=dbType,env_path=env_path)
def get_db_mgr(dbName,dbType):
    env_path = search_env_path(dbName=dbName,dbType=dbType)
    conn_mgr = get_connection_manager(dbName=dbName,dbType=dbType)
    return DatabaseManager(dbName=dbName,dbType=dbType,env_path=env_path,conn_mgr=conn_mgr)
def get_prompt_manager(prompt=None,
                       prompt_data=None,
                           request_data=[''],
                           instruction_data=None,
                           chunk_token_distribution_number=None,
                           completion_percentage=None,
                           instruction_mgr=None,
                           notation=None,
                           model_mgr=None,
                           chunk_number=None,
                           chunk_type=None,
                           model=None):
  prompt = prompt or ''
  prompt_data = prompt_data or []
  request_data = request_data or [prompt]
  instruction_data = instruction_data or []
  chunk_token_distribution_number = chunk_token_distribution_number or 0
  completion_percentage = completion_percentage or 40
  chunk_number=chunk_number or 0
  model_mgr = model_mgr or ModelManager(input_model_name = model)
  instruction_mgr = instruction_mgr or get_instruction_manager(bool_list = [],instructions={})
  chunk_type = chunk_type or "CODE"
  return PromptManager(prompt_data=prompt_data,
                           request_data=[''],
                           instruction_data=[],
                           chunk_token_distribution_number=0,
                           completion_percentage=40,
                           instruction_mgr=instruction_mgr,
                           notation=None,
                           model_mgr=None,
                           chunk_number=chunk_number,
                           chunk_type=chunk_type)
class setup_module():
    def __init__(self,env_key=None,
                   env_path=None,
                   model=None,
                   prompt=None,
                   prompt_percentage=None,
                   prompt_data=[],
                   tableName=None,
                   additional_instructions=None,
                   instruction_bools={}):
        
        self.api_mgr = ApiManager(api_key=env_key,env_path=env_path)
        self.model_mgr = ModelManager(input_model_name = model)
        self.inst_mgr = InstructionManager(additional_instructions,instruction_bools)
        self.prompt_mgr = get_prompt_manager(prompt=prompt,prompt_data=prompt_data,instruction_mgr=self.inst_mgr,model_mgr=self.model_mgr,instruction_data=self.inst_mgr.instructions)
        self.resp_mgr = ResponseManager(prompt_mgr=self.prompt_mgr,api_mgr=self.api_mgr)
        self.response = self.make_query()
    def calculate_extra_tokens(self,prompt=None):
        self.extra_tokens = (self.model_mgr.selected_max_tokens*prompt_percentage) - num_tokens_from_string(prompt_mgr.create_prompt())
        return self.extra_tokens
    async def async_make_query(self):
        response = await self.resp_mgr.initial_query()
        return get_updated_response_content(response)
    def make_query(self):
        return asyncio.run(self.async_make_query())




from abstract_utilities import make_list,get_any_value,safe_read_from_json,write_to_file,safe_json_loads,eatAll
import os
def get_prompt_content(data):
    prompt_data = get_any_value(data,'prompt')
    message_data = get_any_value(prompt_data,'messages')
    if isinstance(message_data,list) and message_data:
        message_data = safe_json_loads(message_data[0])
    return get_any_value(message_data,'content')

def split_prompt_data(prompt_data):
    prompt_parts={}
    prompt_datas = prompt_data.split('-----------------------------------------------------------------------------')
    for prompt_data in prompt_datas[1:]:
        prompt_data = eatAll(prompt_data,['\n','',' ','\t'])
        if prompt_data[0] == '#':
            name = prompt_data[1:].split('#')[0]
            prompt_parts[name]=prompt_data[len(name)+2:]
    
    if 'chunk_data' in prompt_parts:
        chunk_data = eatAll(prompt_parts['chunk_data'],['\n','',' ','\t'])
        if 'this is chunk ' ==chunk_data[:len('this is chunk ')]:
            chunk_num,total_chunks = chunk_data[len('this is chunk '):].split('\n')[0].split(' of ')
            prompt_parts['chunk_num']=int(chunk_num)
            prompt_parts['total_chunks']=int(total_chunks)
    if prompt_parts:
        return prompt_parts
    return {"prompt":prompt_data}
def get_response_content(data):
    response_data = get_any_value(data,'response')
    choice_data = get_any_value(response_data,'choices')
    if isinstance(choice_data,list) and choice_data:
        choice_data = safe_json_loads(choice_data[0])
    return get_any_value(choice_data,'content')

def get_list_of_file_type(list_obj,ext_type):
    return_list=[]
    for file in list_obj:
        baseName,ext=os.path.splitext(file)
        if eatAll(ext,'.') == eatAll(ext_type,'.'):
            return_list.append(file)
    return return_list

def get_data_from_response_file(file_path):
    if file_path:
        if os.path.isfile(file_path):
            data = safe_read_from_json(file_path)
            prompt_content = get_prompt_content(data)
            prompt_pieces = split_prompt_data(prompt_content)
            response_content = get_response_content(data)
            return prompt_pieces,response_content
    return None,None

def clean_list(list_obj):
    while '' in list_obj:
        list_obj.remove('')
    return list_obj
def collate_responses(directory:(str),response_keys:(str or list)=[]):
    response_keys = make_list(response_keys) or ['api_response']
    file_list=[]
    if isinstance(directory,str):
        if os.path.isfile(directory):
            file_list=[directory]
        elif os.path.isdir(directory):
            directory_list = os.listdir(directory)
            for i,file in enumerate(directory_list):
                file_path=file
                if not os.path.isfile(file):
                    file_path = os.path.join(directory,file)
                if os.path.isfile(file_path):
                    file_list.append(new_file_path)
            file_list = get_list_of_file_type(file_list,'json')
    elif isinstance(directory,list):
        file_list=[]
        for path in directory:
            if os.path.isfile(path):
                file_list.append(path)
    description_data={}
    for file_path in file_list:
        baseName,ext=os.path.splitext(file_path)
        prompt_content,response_content = get_data_from_response_file(file_path)
        title = response_content.get('generate_title',baseName)
        if title not in list(description_data.keys()):
            description_data[title]= ['']
            
            for chunk in range(prompt_content.get('total_chunks',1)):
                description_data[title].append('')
        response_data_list=[]
        for response_key in response_keys:
            response_data = get_any_value(response_content,response_key)
            if response_data:
                response_data_list.append(response_data)
        response_num = prompt_content.get('chunk_num',0)
        description_data[title][response_num] = '\n'.join(clean_list(response_data_list))

    for key,values in description_data.items():
        description_data[key]='\n'.join(clean_list(values))
    return description_data
def get_api_response_value(response):
    response = make_list(get_any_value(response,'content') or None)[-1] or response
    response = safe_json_loads(response)
    if not isinstance(response,dict):
        generate_title = "generate_title"
        api_response = "api_response"
        result = response
        if generate_title in result:
            result = result.split(generate_title)[0]
            result = result.split(api_response)[-1]
            result = eatAll(result,[':',',','"',"'",' ','\n','\t'])
    else:
        result = response.get('api_response',response)
    return result
